Package: QuaternaryProd
Type: Package
Title: Computes the Quaternary Dot Product Scoring Statistic for Signed
        and Unsigned Causal Graphs
Version: 1.47.0
Date: 2015-10-22
Authors@R: c(
    person("Carl Tony", "Fakhry", email = "cfakhry@cs.umb.edu", role = c("cre","aut")),
    person("Ping", "Chen", role = "ths"),
    person("Kourosh", "Zarringhalam", role = c("aut","ths")))
Description: QuaternaryProd is an R package that performs causal reasoning on biological 
             networks, including publicly available networks such as STRINGdb. QuaternaryProd 
             is an open-source alternative to commercial products such as Inginuity Pathway 
             Analysis. For a given a set of differentially expressed genes, QuaternaryProd 
             computes the significance of upstream regulators in the network by performing causal
             reasoning using the Quaternary Dot Product Scoring Statistic (Quaternary Statistic),
             Ternary Dot product Scoring Statistic (Ternary Statistic) and Fisher's exact test
	     (Enrichment test). The Quaternary Statistic handles signed, unsigned and ambiguous
             edges in the network. Ambiguity arises when the direction of causality is unknown,
             or when the source node (e.g., a protein) has edges with conflicting signs for the 
	     same target gene. On the other hand, the Ternary Statistic provides causal reasoning
             using the signed and unambiguous edges only. The Vignette provides more details on the
             Quaternary Statistic and illustrates an example of how to perform causal reasoning
             using STRINGdb. 
License: GPL (>=3)
biocViews: GraphAndNetwork, GeneExpression, Transcription
Depends: R (>= 3.2.0), Rcpp (>= 0.11.3), dplyr, yaml (>= 2.1.18)
Suggests: knitr
LinkingTo: Rcpp
VignetteBuilder: knitr
RoxygenNote: 6.1.0
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/QuaternaryProd
git_branch: devel
git_last_commit: b357c3c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: yes
Packaged: 2026-05-18 23:09:11 UTC; biocbuild
Author: Carl Tony Fakhry [cre, aut],
  Ping Chen [ths],
  Kourosh Zarringhalam [aut, ths]
Maintainer: Carl Tony Fakhry <cfakhry@cs.umb.edu>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-19 05:04:32 UTC; unix

# Generated by roxygen2: do not edit by hand

export(QP_Pmf)
export(QP_Probability)
export(QP_Pvalue)
export(QP_SigPvalue)
export(QP_Support)
export(RunCRE_HSAStringDB)
import(dplyr)
import(yaml)
importFrom(Rcpp,evalCpp)
importFrom(stats,complete.cases)
importFrom(stats,na.omit)
importFrom(utils,read.table)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
useDynLib(QuaternaryProd)

# QuaternaryProd
An R package for computing the Quaternary Dot Product Scoring Statistic for signed and unsigned causal graphs.

## Installation
Before installing this package, make sure you have the latest version of *Rstudio*, *R* and the *devtools* package. You can install this R pacakge using the following:
```{R}
library(devtools)
install_github("carltonyfakhry/QuaternaryProd", build_vignettes = TRUE, local = FALSE)
```
## Usage
For an introduction to the Quaternary Dot Product Scoring Statistic and for an example on how to compute it over the publicly available network *STRINGdb*, please see 
the *Vignette* for this package using the following:
```{R}
browseVignettes("QuaternaryProd")
```

## ----message=FALSE------------------------------------------------------------
library(QuaternaryProd)

# Compute the probability mass function
pmf <- QP_Pmf(q_p = 20, q_m = 20, q_z = 20, q_r = 0, n_p = 20, n_m = 20, n_z = 20)

# Plot the mass function
plot(names(pmf), pmf, col="blue", xlab = "scores", ylab = "probabilities")
lines(names(pmf), pmf, col = "blue")

## ----message=FALSE------------------------------------------------------------
# Get the p-value of score 5
pval <- QP_Pvalue(score = 5, q_p = 20, q_m = 20, q_z = 20, q_r = 0, 
                                                     n_p = 20, n_m = 20, n_z = 20)
pval

# Compue the p-value only if it is statistically significant otherwise
# return -1
pval <- QP_SigPvalue(score = 5, q_p = 20, q_m = 20, q_z = 20, q_r = 0, 
                                                     n_p = 20, n_m = 20, n_z = 20)
pval

## ----message=FALSE------------------------------------------------------------
library(QuaternaryProd)

# Get gene expression data
e2f3 <- system.file("extdata", "e2f3_sig.txt",
                             package = "QuaternaryProd")
e2f3 <- read.table(e2f3, sep = "\t",
                             header = TRUE, stringsAsFactors = FALSE)
myc <- system.file("extdata", "myc_sig.txt",
                             package = "QuaternaryProd")
myc <- read.table(myc, sep = "\t",
                             header = TRUE, stringsAsFactors = FALSE)

# Rename column names appropriately
# and remove duplicated entrez ids in the gene expression data
names(e2f3) <- c("entrez", "pvalue", "fc")
e2f3 <- e2f3[!duplicated(e2f3$entrez),]

names(myc) <- c("entrez", "pvalue", "fc")
myc <- myc[!duplicated(myc$entrez),]

## ----message=FALSE, progress=FALSE--------------------------------------------
# Compute the Quaternary Dot Product Scoring Statistic for only statistically
# significant regulators
quaternary_results <- RunCRE_HSAStringDB(e2f3, method = "Quaternary",
                                   fc.thresh = log2(1.3), pval.thresh = 0.05,
                                   only.significant.pvalues = TRUE,
                                   significance.level = 0.05,
                                   epsilon = 1e-16, progressBar = FALSE,
                                   relations = NULL, entities = NULL)
quaternary_results[1:4, c("uid","symbol","regulation","pvalue")]

## ----message=FALSE, progress=FALSE--------------------------------------------
ternary_results <- RunCRE_HSAStringDB(myc, method = "Ternary",
                                      fc.thresh = log2(1.3), pval.thresh = 0.05,
                                      only.significant.pvalues = TRUE,
                                      significance.level = 0.05,
                                      epsilon = 1e-16, progressBar = FALSE,
                                      relations = NULL, entities = NULL)
ternary_results[1:4, c("uid","symbol","regulation","pvalue")]

## ----message=FALSE, progress=TRUE, quiet = TRUE-------------------------------
enrichment_results <- RunCRE_HSAStringDB(myc, method = "Enrichment",
                                         fc.thresh = log2(1.3), pval.thresh = 0.05,
                                         only.significant.pvalues = TRUE,
                                         significance.level = 0.05,
                                         epsilon = 1e-16, progressBar = FALSE,
                                         relations = NULL, entities = NULL)
enrichment_results[enrichment_results$symbol == "MYC", c("symbol","regulation","pvalue")]

# Get the value of regulation of a trguid from the gene expression
# data (i.e in evidence)
getGeneVals <- function(trguids, gene_expression_data){
  trguids <- as.vector(trguids) 
  val <- rep(0, length(trguids))
  ind <- match(trguids, gene_expression_data[,"uid"])
  ind2 <- na.omit(ind)
  if(length(ind2) > 0){
    val[which(!is.na(ind))] <- gene_expression_data[,"val"][ind2]
  }
  return(val)
}



# This function checks the validity of the relations data frame
check_relations <- function(relations, entities){
  
  if(ncol(relations) != 3 | !all(names(relations) == c("srcuid", "trguid", "mode"))){
    stop("Please provide a valid relations data frame containing columns: srcuid, trguid, mode!")
  }
    
  # Check for NAs in relations
  if (length(which(is.na(relations))) > 0){
    stop("Relations data frame contains rows with NAs!")
  }
  
  # Check if there are duplicated rows in relations
  if(anyDuplicated(relations)){
    stop("Duplicate rows in relations found!")
  }
  
  # Check if there are duplicate srcuid-trguid relations with different modes
  if(anyDuplicated(relations[,c("srcuid", "trguid")])){
    stop("Duplicate srcuid-trguid relation with different modes found in relations!")
  }
  
  # Check data types of entities column
  if(!is.integer(relations$srcuid) | !is.integer(relations$trguid) | !is.integer(relations$mode)){
    stop("In relations, all columns must be of type integer!")
  }
  
  # Make sure all relations are either increases, decreases or regulates
  if(!all(relations$mode %in% c(1, -1, 0))){
    stop("All relations between srcuid and trguid in relations data frame must be either +1, -1 or 2!")
  }
  
  # Make sure all relation nodes are in entities
  unique.srcuids <- unique(relations$srcuid)
  unique.trguids <- unique(relations$trguid)
  
  if(!all(unique.trguids %in% entities$uid)){
    stop("All trguids in relations must be present in the uid column of the entities data frame!")
  }
  
  if(!all(unique.srcuids %in% entities$uid)){
    stop("All srcuids in relations must be present in the uid column of the entities data frame!")
  }
  
}



# This function checks the validity of the entities data frame
check_entities <- function(entities){

  if(ncol(entities) != 4 | !all(names(entities) == c("uid", "id", "ensembleid", "symbol"))){
    stop("Please provide a valid entities data frame containing columns: uid, id, ensembleid and symbol!")
  }
    
  # Check for NAs in entities
  if(length(which(is.na(entities))) > 0){
    stop("Entities data frame contains rows with NAs!")
  }
  
  # Check if there are duplicated rows in entities
  if(nrow(entities[duplicated(entities),]) > 0){
    stop("Duplicate rows in entities found!")
  }
  
  # Check if there are duplicated uids in entities
  if(any(duplicated(entities$uid))){
    stop("Please remove duplicated uid in entities data frame!")
  }
  
  # Check data types of entities column
  if(!is.integer(entities$uid)){
    stop("In entities, column uid must be of type integer!")
  }
  
  if(!is.character(entities$id) | !is.character(entities$symbol) | !is.character(entities$ensembleid)){
    stop("In entities, columns id, ensembleid and symbol must be of type character!")
  }
  

}



#' This function runs a causal relation engine by computing the Quaternary Dot
#' Product Scoring Statistic, Ternary Dot Product Scoring Statistic or the Enrichment test over the Homo
#' Sapien STRINGdb causal network (version 10 provided under the Creative Commons license: 
#' https://creativecommons.org/licenses/by/3.0/). Note that the user has the option of specifying other causal networks
#' with this function.
#' 
#' @description This function runs a causal relation engine by computing the Quaternary Dot
#'              Product Scoring Statistic, Ternary Dot Product Scoring Statistic or the Enrichment test over the Homo
#'              Sapien STRINGdb causal network (version 10 provided under the Creative Commons license: 
#'              https://creativecommons.org/licenses/by/3.0/). Note that the user has the option of specifying other causal networks
#'              with this function.
#' 
#' @usage RunCRE_HSAStringDB(gene_expression_data, method = "Quaternary", 
#'                     fc.thresh = log2(1.3), pval.thresh = 0.05, 
#'                     only.significant.pvalues = FALSE, 
#'                     significance.level = 0.05,
#'                     epsilon = 1e-16, progressBar = TRUE, 
#'                     relations = NULL, entities = NULL) 
#' 
#' @param gene_expression_data A data frame for gene expression data. The \code{gene_expression_data} data frame must have three columns \code{entrez}, 
#'        \code{fc} and \code{pvalue}. \code{entrez} denotes the entrez id of a given gene, \code{fc} denotes
#'        the fold change of a gene, and \code{pvalue} denotes the p-value. The \code{entrez} column must be of type
#'        integer or character, and the \code{fc} and \code{pvalue} columns must be numeric values.
#' @param method Choose one of \code{Quaternary}, \code{Ternary} or \code{Enrichment}. Default is \code{Quaternary}.
#' @param fc.thresh Threshold for fold change in \code{gene_expression_data} data frame. Any row in gene_expression_data with abosolute value of \code{fc}
#'        smaller than \code{fc.thresh} will be ignored. Default value is \code{fc.thresh = log2(1.3)}. 
#' @param pval.thresh Threshold for p-values in \code{gene_expression_data} data frame. All rows in \code{gene_expression_data} with p-values 
#'        greater than \code{pval.thresh} will be ingnored. Default value is \code{pval.thresh = 0.05}. 
#' @param only.significant.pvalues If \code{only.significant.pvalues = TRUE} then only p-values for statistically significant regulators
#'        are computed otherwise uncomputed p-values are set to -1. The default value is \code{only.significant.pvalues = FALSE}.
#' @param significance.level When \code{only.significant.pvalues = TRUE}, only p-values which are less than or equal to 
#'                           \code{significance.level} are computed. The default value is \code{significance.level = 0.05}.
#' @param epsilon Threshold for probabilities of matrices. Default value is \code{threshold = 1e-16}.
#' @param progressBar Progress bar for the percentage of computed p-values for the regulators in the network. Default
#'        value is \code{progressBar = TRUE}. 
#' @param relations A data frame containing pairs of connected entities in a causal network,
#'        and the type of causal relation between them. The data frame must have three columns with column names: \emph{srcuid}, 
#'        \emph{trguid} and \emph{mode} respective of order. \emph{srcuid} stands for source entity, \emph{trguid} stands for 
#'        target entity and \emph{mode} stands for the type of relation between \emph{srcuid} and \emph{trguid}. The relation 
#'        has to be one of \emph{+1} for \emph{upregulation}, \emph{-1} for \emph{downregulation} or \emph{0} for regulation without
#'        specified direction of regulation. All three columns must be of type integer. Default value is \code{relations = NULL}.
#' @param entities A data frame of mappings for all entities present in data frame \emph{relations}. \emph{entities} must contain
#'        four columns: \emph{uid}, \emph{id}, \emph{ensembleid} and \emph{symbol} respective of order. \emph{uid} must be 
#'        of type integer and \emph{id}, \emph{ensembleid} and \emph{symbol} must be of type character. \emph{uid} includes every source and target 
#'        node in the network (i.e \emph{relations}),
#'        \emph{id} is the id of \emph{uid} (e.g entrez id of an mRNA), \emph{ensembleid} is the ensemble id of \emph{id} and \emph{symbol} 
#'        is the symbol of \emph{id}. Default value is \code{entities = NULL}.
#'           
#' @return This function returns a data frame containing parameters concerning the method used. The p-values of each
#'         of the regulators is also computed, and the data frame
#'         is in increasing order of p-values of the goodness of fit score for the given regulators. The column
#'         names of the data frame are:
#'         
#' \itemize{        
#' \item  \code{uid} The regulator in the causal network.
#' \item \code{symbol} Symbol of the regulator. 
#' \item \code{regulation} Direction of regulation of the regulator.
#' \item \code{correct.pred} Number of correct predictions in \code{gene_expression_data} when compared to predictions made
#'                     by the network.
#' \item \code{incorrect.pred} Number of incorrect predictions in \code{gene_expression_data} when compared to predictions made
#'                     by the network.
#' \item \code{score} The number of correct predictions minus the number of incorrect predictions. 
#' \item \code{total.reachable} Total Number of children of the given regulator.
#' \item \code{significant.reachable} Number of children of the given regulator that are also present 
#'                                    in \code{gene_expression_data}.
#' \item \code{total.ambiguous} Total number of children of the given regulator which are regulated by the given regulator without
#'                              knowing the direction of regulation.
#' \item \code{significant.ambiguous} Total number of children of the given regulator which are regulated by the given regulator without
#'                              knowing the direction of regulation and are also present in \code{gene_expression_data}.  
#' \item \code{unknown} Number of target nodes in the causal network which do not interact with the given regulator.
#' \item \code{pvalue} P-value of the score computed according to the selected method. If \code{only.significant.pvalues = TRUE}
#'                     and the \code{pvalue} of the regulator is greater than \code{significance.level}, then
#'                     the p-value is not computed and is set to a value of -1.
#' }
#' 
#' @author Carl Tony Fakhry, Ping Chen and Kourosh Zarringhalam
#' 
#' @references Carl Tony Fakhry, Parul Choudhary, Alex Gutteridge, Ben Sidders, Ping Chen, Daniel Ziemek, and
#'             Kourosh Zarringhalam. Interpreting transcriptional changes using causal graphs: new methods and
#'             their practical utility on public networks. BMC Bioinformatics, 17:318, 2016. ISSN 1471-2105.
#'             doi: 10.1186/s12859-016-1181-8.
#'            
#'             Franceschini, A (2013). STRING v9.1: protein-protein interaction networks, with increased coverage 
#'             and integration. In:'Nucleic Acids Res. 2013 Jan;41(Database issue):D808-15. doi: 10.1093/nar/gks1094. 
#'             Epub 2012 Nov 29'.
#'             
#' @examples 
#'
#' # Get gene expression data
#' e2f3 <- system.file("extdata", "e2f3_sig.txt", package = "QuaternaryProd")
#' e2f3 <- read.table(e2f3, sep = "\t", header = TRUE, stringsAsFactors = FALSE)
#' 
#' # Rename column names appropriately and remove duplicated entrez ids
#' names(e2f3) <- c("entrez", "pvalue", "fc")
#' e2f3 <- e2f3[!duplicated(e2f3$entrez),]
#' 
#' # Compute the Quaternary Dot Product Scoring statistic for statistically significant
#' # regulators in the STRINGdb network
#' enrichment_results <- RunCRE_HSAStringDB(e2f3, method = "Enrichment",
#'                              fc.thresh = log2(1.3), pval.thresh = 0.05,
#'                              only.significant.pvalues = TRUE)
#' enrichment_results[1:4, c("uid","symbol","regulation","pvalue")]
#'
#' @export

RunCRE_HSAStringDB <- function(gene_expression_data, method = "Quaternary", 
                               fc.thresh = log2(1.3), pval.thresh = 0.05,
                               only.significant.pvalues = FALSE,
                               significance.level = 0.05,
                               epsilon = 1e-16, progressBar = TRUE, 
                               relations = NULL, entities = NULL){
  
  cat("(1/5) Begin Processing...\n")
  
  # Check method
  if(!(method %in% c("Quaternary", "Ternary", "Enrichment"))){
    stop("Method must be one of Quaternary, Ternary or Enrichment!")
  }
  
  # Check fc.thresh
  if(length(fc.thresh) != 1 || !is.numeric(fc.thresh)){
    stop("fc.thresh should be a numeric value!")
  }
  
  # Check pval.thresh
  if(length(pval.thresh) != 1 || !is.numeric(pval.thresh) | pval.thresh > 1 | pval.thresh < 0){
    stop("pval.thresh should be a numeric value >= 0 and <= 1!")
  }
  
  # Check only.significant.pvalues
  if(length(only.significant.pvalues) != 1 | !is.logical(only.significant.pvalues)){
    stop("only.significant.pvalues must be set to either TRUE or FALSE!")
  }
  
  # Check significance.level
  if(length(significance.level) != 1 || !is.numeric(significance.level) | significance.level > 1 | significance.level < 0){
    stop("significance.level should be a numeric value >= 0 and <= 1!")
  }
  
  # Epsilon must be positive between 0 and 1.
  if (length(epsilon) != 1 || !is.numeric(epsilon) || epsilon < 0 || epsilon > 1){
    stop("Epsilon must be positive numeric number >= 0 and <= 1!\n")
  }
  
  # progressBar must be logical value.
  if (length(progressBar) != 1 || !is.logical(progressBar)){
    stop("progressBar must be a logical value!")
  }
  
  # Check relations and entities
  usingSTRINGdb <- NULL
  if((length(relations) != 0 & !is.null(relations)) & (length(entities) != 0 & !is.null(entities))){
    check_relations(relations, entities)
    check_entities(entities)
    usingSTRINGdb <- FALSE
  }else if(length(relations) == 0 & is.null(relations) & length(entities) == 0 & is.null(entities)){ 
    f1 <- system.file("extdata", "StringRels.dat", package="QuaternaryProd")
    relations <- read.table(f1, header = T, stringsAsFactors = F)
    f2 <- system.file("extdata", "StringEnts.dat", package="QuaternaryProd")
    entities <- read.table(f2, header = T, stringsAsFactors = F)
    usingSTRINGdb <- TRUE
  }else{
    stop("Both relations and entities must be specified otherwise set relations and entities to NULL and use STRINGdb!")
  }
      
  
  if (!is.data.frame(gene_expression_data)){ 
    stop("Please enter a data frame for gene_expression_data!")
  }
  gene_expression_data <- data.frame(gene_expression_data)
  
  
  if (nrow(gene_expression_data) == 0){
    stop("Please enter a non-empty data frame for gene_expression_data!")
  }
  
  # Do more validation for gene_expression_data data frame
  if(ncol(gene_expression_data) == 3){
 
    # Check for NAs in gene_expression_data
    if (length(which(is.na(gene_expression_data))) > 0){
      stop("gene_expression_data data frame contains rows with NAs. Please only provide complete cases!")
    }
        
    # Check if there are duplicated rows in gene_expression_data
    if(nrow(gene_expression_data[duplicated(gene_expression_data),]) > 0){
      stop("Duplicate rows in gene_expression_data found. Please remove duplicate rows!")
    }
    
    # Check for correct column names
    pval.ind <- which(colnames(gene_expression_data) %in% c("pvalue"))
    fc.ind <- which(colnames(gene_expression_data) %in% c("fc") )
    id.ind <- which(colnames(gene_expression_data) %in% c("entrez"))
    
    # Stop if column names are not correct
    if(length(id.ind) == 0 | length(fc.ind) == 0 | length(pval.ind) == 0){
      stop("Please label gene_expression_data column names as entrez, fc, pvalue!")
    }
    
    # Rearrange data
    gene_expression_data <- gene_expression_data[,c(id.ind, fc.ind, pval.ind)]
    names(gene_expression_data) <- c("entrez", "fc", "pvalue")
    
    # Check data types of entities column
    n.e0 <- nrow(gene_expression_data)
    if(is.character(gene_expression_data[,"entrez"])){
      suppressWarnings(gene_expression_data[,"entrez"] <- as.double(gene_expression_data[,"entrez"]))
      gene_expression_data <- gene_expression_data[complete.cases(gene_expression_data),]
      n.e0 <- n.e0 - nrow(gene_expression_data)
      if(any(gene_expression_data[,"entrez"] != floor(gene_expression_data[,"entrez"]))){
        stop("None of the entrez ids in gene_expression_data can be a double or a character representation of a double!")
      }
      gene_expression_data[,"entrez"] <- as.integer(gene_expression_data[,"entrez"])
    }else if(is.integer(gene_expression_data[,"entrez"])){
    }else{
      stop("In gene_expression_data, column entrez must be of type character or integer (but not a factor)!")
    }
    
    if(!is.numeric(gene_expression_data[,"fc"])){
      stop("In gene_expression_data, column fc must be numeric!")
    }
    
    if(!is.numeric(gene_expression_data[,"pvalue"])){
      stop("In gene_expression_data, column pvalue must be numeric!")
    }

    # Make sure p-values are positive and less than 1
    if(!all(gene_expression_data[,"pvalue"] >= 0)){
      stop("All P-values in gene_expression_data data frame must be greater or equal to zero!")
    }
    
    if(!all(gene_expression_data[,"pvalue"] <= 1)){
      stop("All P-values in gene_expression_data data frame must be smaller than one!")
    }
    
    # Remove duplicate entrez in gene_expression_data
    if(any(duplicated(gene_expression_data[,"entrez"]))){
      stop("Remove duplicated entrez ids in gene_expression_data!")
    }
    
    # Filter by p-value and fold-change
    gene_expression_data <- gene_expression_data[abs(gene_expression_data[,"fc"]) >= fc.thresh & gene_expression_data[,"pvalue"] <= pval.thresh,]
    gene_expression_data <- gene_expression_data[,c("entrez","fc")]
    
    # Check if any gene_expression_data left
    if(nrow(gene_expression_data) == 0){
      stop("No rows left in gene_expression_data after rows are filtered according to fc.thresh and pval.thresh!")
    }
    
    # Where fc is greater than 0 replace with increase i.e 1 and where fc is negative
    # replace with -1 i.e decreases
    gene_expression_data[,2] <- ifelse(gene_expression_data[,2] > 0, 1, -1)
    names(gene_expression_data) <- c("entrez", "val")
    
  } else{
    
    stop("Please provide a valid gene_expression_data data frame containing columns: entrez, fc, pvalue")
  
  }
  
  # Make sure all gene_expression_data entrez are in entities ids
  n.e1 <- nrow(gene_expression_data)
  gene_expression_data <- gene_expression_data[gene_expression_data[,"entrez"] %in% entities[,"id"],]
  n.e2 <- nrow(gene_expression_data)
  if (n.e2 == 0){
    stop("All entrez ids in gene_expression_data are not present in entities data frame!")
  }
  cat(paste("(2/5)", (n.e1-n.e2), "rows from gene_expression_data removed due", "\n", "\t\t to entrez ids being unrepsented in network entities!\n", sep = " "))
  
  # Make sure no entrez id in entities maps to two entities
  n.e1 <- nrow(gene_expression_data)
  gene_expression_data.tmp <- merge(gene_expression_data, entities, by.x = "entrez" , by.y = "id")
  n.e2 <- nrow(gene_expression_data.tmp)
  if(n.e2 > n.e1){
    stop("At least one entrez id in gene_expression_data maps to more than one id in network entities!")
  }
  gene_expression_data <- data.frame(uid = gene_expression_data.tmp[,"uid"], val = gene_expression_data.tmp[,"val"], stringsAsFactors = F)
  
  u.hyps <- NULL
  child.uid <- NULL
  child.sgn <- NULL
  if(!usingSTRINGdb){
    cat("(3/5) Processing specified causal network...\n")
    u.hyps <- unique(relations$srcuid)
    child.uid <- lapply(u.hyps, function(x, relations) relations$trguid[which(relations$srcuid == x)], relations = relations)
    child.sgn <- lapply(u.hyps, function(x, relations) ifelse(relations$mode[which(relations$srcuid == x)] == 1,
                                                              1, ifelse(relations$mode[which(relations$srcuid == x)] == -1, -1, 0)), relations = relations)
  }else{
    cat("(3/5) Loading STRINGdb causal network...\n")
    u.hyps <- read_yaml(system.file("extdata", "u.hyps.yaml", package="QuaternaryProd"))
    child.uid <- read_yaml(system.file("extdata", "child.uid.yaml", package="QuaternaryProd"))
    child.sgn <- read_yaml(system.file("extdata", "child.sgn.yaml", package="QuaternaryProd"))
  }
  
  # Get the value of regulation of the children from the gene expression data
  child.val <- lapply(child.uid, function(x, gene_expression_data) 
                      getGeneVals(x, gene_expression_data), gene_expression_data = gene_expression_data)

  # Get gene values from gene_expression_data for all non children
  unique.children <- unique(relations$trguid)
  non.child.val <- lapply(child.uid, function(x, gene_expression_data, unique.children) 
                          getGeneVals(unique.children[which(!(unique.children %in% x))], gene_expression_data), 
                          gene_expression_data = gene_expression_data, unique.children = unique.children)
  
  results <- data.frame(matrix(0, nrow  = 2 * length(u.hyps), ncol = 12), stringsAsFactors = F)
  colnames(results) <- c('uid', 'symbol', 'regulation', 'correct.pred', 'incorrect.pred', 'score',
                        'total.reachable', 'significant.reachable', 'total.ambiguous', 'significant.ambiguous',
                        'unknown', 'pvalue')
  
  cat("(4/5) Computing P-values...\n")
  if(progressBar){
    pb <- txtProgressBar(min = 1, max = length(u.hyps), style = 3)
  }
  
  for(p.s in 1:length(u.hyps)){

    results[(2*(p.s-1)+1), 1] <- entities$ensembleid[which(entities$uid == u.hyps[p.s])]
    results[(2*p.s), 1]       <- entities$ensembleid[which(entities$uid == u.hyps[p.s])]
    results[(2*(p.s-1)+1), 2] <- entities$symbol[which(entities$uid == u.hyps[p.s])]
    results[(2*p.s), 2]       <- entities$symbol[which(entities$uid == u.hyps[p.s])]
    results[(2*(p.s-1)+1), 3] <- 'up'
    results[(2*p.s), 3]       <- 'down'
    
    npp = length(which(child.sgn[[p.s]] == 1 &  child.val[[p.s]] == 1))
    npm = length(which(child.sgn[[p.s]] == 1 &  child.val[[p.s]] == -1))
    npz = length(which(child.sgn[[p.s]] == 1 &  child.val[[p.s]] == 0))
    
    nmp = length(which(child.sgn[[p.s]] == -1 &  child.val[[p.s]] == 1))
    nmm = length(which(child.sgn[[p.s]] == -1 &  child.val[[p.s]] == -1))
    nmz = length(which(child.sgn[[p.s]] == -1 &  child.val[[p.s]] == 0))
    
    nrp = length(which(child.sgn[[p.s]] == 0 &  child.val[[p.s]] == 1))
    nrm = length(which(child.sgn[[p.s]] == 0 &  child.val[[p.s]] == -1))
    nrz = length(which(child.sgn[[p.s]] == 0 &  child.val[[p.s]] == 0))
    
    nzp = length(which(non.child.val[[p.s]] == 1))
    nzm = length(which(non.child.val[[p.s]] == -1))
    nzz = length(which(non.child.val[[p.s]] == 0))
    
    if (method == "Quaternary"){
      pval <- runCRE(npp, npm, npz, nmp, nmm, nmz, nrp, nrm, nrz, nzp, nzm, nzz, method = 'Quaternary',
                     only.significant.pvalues = only.significant.pvalues, significance.level = significance.level, epsilon = epsilon)
    } else if (method == "Ternary"){
      pval <- runCRE(npp, npm, npz, nmp, nmm, nmz, nrp, nrm, nrz, nzp, nzm, nzz, method = 'Ternary',
                     only.significant.pvalues = only.significant.pvalues, significance.level = significance.level, epsilon = epsilon)
    } else if (method == "Enrichment"){
      pval <- runCRE(npp, npm, npz, nmp, nmm, nmz, nrp, nrm, nrz, nzp, nzm, nzz, method = 'Enrichment',
                     only.significant.pvalues = only.significant.pvalues, significance.level = significance.level, epsilon = epsilon)
    } else {
      stop("Select one of methods: Quaternary, Ternary or Enrichment")
    }
    
    qPlus  <- npp + npm + npz
    qMinus <- nmp + nmm + nmz
    qR     <- nrp + nrm + nrz
    qZero  <- nzp + nzm + nzz
    
    results[(2*(p.s-1)+1), 4]  <- npp + nmm
    results[(2*(p.s-1)+1), 5]  <- npm + nmp
    results[(2*(p.s-1)+1), 6]  <- npp + nmm - (npm + nmp)
    results[(2*(p.s-1)+1), 7]  <- qPlus + qMinus + qR
    results[(2*(p.s-1)+1), 8]  <- npp + npm + nmp + nmm + nrp + nrm
    results[(2*(p.s-1)+1), 9]  <- qR
    results[(2*(p.s-1)+1), 10] <- nrp + nrm
    results[(2*(p.s-1)+1), 11] <- qZero
    results[(2*(p.s-1)+1), 12] <- pval$pval.up
    
    results[(2*p.s), 4]  <- nmp + npm
    results[(2*p.s), 5]  <- npp + nmm
    results[(2*p.s), 6]  <- nmp + npm - (npp + nmm)
    results[(2*p.s), 7]  <- qPlus + qMinus + qR
    results[(2*p.s), 8]  <- npp + npm + nmp + nmm + nrp + nrm
    results[(2*p.s), 9]  <- qR
    results[(2*p.s), 10] <- nrp + nrm
    results[(2*p.s), 11] <- qZero
    results[(2*p.s), 12] <- pval$pval.down
  
    if(progressBar){  
      setTxtProgressBar(pb, p.s)
    }
    
  }
  
  if(progressBar){
    close(pb)
  }  
    
  results.tmp1 <- results[which(results$pvalue > -1),]
  results.tmp1 <- results.tmp1[order(results.tmp1$pvalue),]
  results.tmp2 <- results[which(results$pvalue == -1),]
  results <- rbind(results.tmp1, results.tmp2)
  rownames(results) <- 1:nrow(results)
  cat("(5/5) Done.\n")
  
  return(results)
  
}  



# This function runs the CRE based on the version specified
runCRE = function(npp, npm, npz, nmp, nmm, nmz, nrp, nrm, nrz, nzp, nzm, nzz, method, only.significant.pvalues, significance.level, epsilon){
  
  if(method == 'Quaternary'){
    
    qR     <- nrp + nrm + nrz
    qZero  <- nzp + nzm + nzz
    nPlus  <- npp + nmp + nrp + nzp
    nMinus <- npm + nmm + nrm + nzm
    nZero  <- npz + nmz + nrz + nzz
    
    ## Assume up-regulated
    qPlus  <- npp + npm + npz
    qMinus <- nmp + nmm + nmz
    score  <- npp + nmm + nrp + nrm - (npm + nmp)
    pval.up <- NULL
    if(only.significant.pvalues){
      pval.up <- QP_SigPvalue(score = score, q_p = qPlus, q_m = qMinus, q_z = qZero,
                           q_r = qR, n_p = nPlus, n_m = nMinus, n_z = nZero, epsilon = epsilon, sig_level = significance.level)
    }else{
      pval.up <- QP_Pvalue(score = score, q_p = qPlus, q_m = qMinus, q_z = qZero,
                          q_r = qR, n_p = nPlus, n_m = nMinus, n_z = nZero, epsilon = epsilon)
    }
    
    ## Assume down-regulated
    qPlus  <- nmp + nmm + nmz
    qMinus <- npp + npm + npz
    score  <- nmp + npm + nrp + nrm - (npp + nmm)
    pval.down <- NULL
    if(only.significant.pvalues){
      pval.down <- QP_SigPvalue(score = score, q_p = qPlus, q_m = qMinus, q_z = qZero,
                            q_r = qR, n_p = nPlus, n_m = nMinus, n_z = nZero, epsilon = epsilon, sig_level = significance.level)
    }else{
      pval.down <- QP_Pvalue(score = score, q_p = qPlus, q_m = qMinus, q_z = qZero,
                             q_r = qR, n_p = nPlus, n_m = nMinus, n_z = nZero, epsilon = epsilon)
    }
    
  } else if(method == 'Ternary'){
    qR     <- 0
    qZero  <- nzp + nzm + nzz
    nPlus  <- npp + nmp + nzp
    nMinus <- npm + nmm + nzm
    nZero  <- npz + nmz + nzz
    
    ## Assume up-regulated
    qPlus  <- npp + npm + npz
    qMinus <- nmp + nmm + nmz
    score  <- npp + nmm - (npm + nmp)
    pval.up <- NULL
    if(only.significant.pvalues){
      pval.up <- QP_SigPvalue(score = score, q_p = qPlus, q_m = qMinus, q_z = qZero,
                              q_r = qR, n_p = nPlus, n_m = nMinus, n_z = nZero, epsilon = epsilon, sig_level = significance.level)
    }else{
      pval.up <- QP_Pvalue(score = score, q_p = qPlus, q_m = qMinus, q_z = qZero,
                           q_r = qR, n_p = nPlus, n_m = nMinus, n_z = nZero, epsilon = epsilon)
    }
    
    ## Assume down-regulated
    qPlus  <- nmp + nmm + nmz
    qMinus <- npp + npm + npz
    score  <- nmp + npm - (npp + nmm)
    pval.down <- NULL
    if(only.significant.pvalues){
      pval.down <- QP_SigPvalue(score = score, q_p = qPlus, q_m = qMinus, q_z = qZero,
                                q_r = qR, n_p = nPlus, n_m = nMinus, n_z = nZero, epsilon = epsilon, sig_level = significance.level)
    }else{
      pval.down <- QP_Pvalue(score = score, q_p = qPlus, q_m = qMinus, q_z = qZero,
                             q_r = qR, n_p = nPlus, n_m = nMinus, n_z = nZero, epsilon = epsilon)
    }
    
    
  } else if(method == 'Enrichment'){
    
    nrp    <- npp + nmp + nrp
    nrm    <- npm + nmm + nrm
    nrz    <- npz + nmz + nrz
    
    qPlus  <- 0
    qMinus <- 0
    qR     <- nrp + nrm + nrz
    qZero  <- nzp + nzm + nzz
    
    nPlus  <- nrp + nzp
    nMinus <- nrm + nzm
    nZero  <- nrz + nzz
    
    score  <- nrp + nrm
    
    if(only.significant.pvalues){
      pval.up <- QP_SigPvalue(score = score, q_p = qPlus, q_m = qMinus, q_z = qZero,
                              q_r = qR, n_p = nPlus, n_m = nMinus, n_z = nZero, epsilon = epsilon, sig_level = significance.level)
    }else{
      pval.up <- QP_Pvalue(score = score, q_p = qPlus, q_m = qMinus, q_z = qZero,
                           q_r = qR, n_p = nPlus, n_m = nMinus, n_z = nZero, epsilon = epsilon)
    }
    
    pval.down <- pval.up
    
  }
  
  pval <- list(pval.up = pval.up, pval.down = pval.down)
  
}

#' Computes the support for the scores.
#' 
#' @description This function computes the support of the Quaternary Dot Product Scoring distribution
#' for signed causal graphs. This includes all scores which have probabilities strictly greater than 0.
#' 
#' @usage QP_Support(q_p, q_m, q_z, q_r, n_p, n_m, n_z)
#' 
#' @param q_p Expected number of positive predictions.
#' @param q_m Expected number of negative predictions.
#' @param q_z Expected number of nil predictions.
#' @param q_r Expected number of regulated predictions. 
#'
#' @param n_p Number of positive predictions from experiments.
#' @param n_m Number of negative predictions from experiments.
#' @param n_z Number of nil predictions from experiments. 
#'        
#' @return Integer vector of support.
#' 
#' @author Carl Tony Fakhry, Ping Chen and Kourosh Zarringhalam 
#' 
#' @references Carl Tony Fakhry, Parul Choudhary, Alex Gutteridge, Ben Sidders, Ping Chen, Daniel Ziemek, and
#'             Kourosh Zarringhalam. Interpreting transcriptional changes using causal graphs: new methods and
#'             their practical utility on public networks. BMC Bioinformatics, 17:318, 2016. ISSN 1471-2105.
#'             doi: 10.1186/s12859-016-1181-8.
#'            
#'             Franceschini, A (2013). STRING v9.1: protein-protein interaction networks, with increased coverage 
#'             and integration. In:'Nucleic Acids Res. 2013 Jan;41(Database issue):D808-15. doi: 10.1093/nar/gks1094. 
#'             Epub 2012 Nov 29'.
#'             
#' @examples # Compute the support of the Quaternary Dot Product Scoring distribution with the given margins.
#' QP_Support(50,50,50,0,50,50,50)
#' 
#' @export

QP_Support <- function(q_p, q_m, q_z, q_r, n_p, n_m, n_z){
  
  # All row constraints must be numbers.
  if (!is.numeric(q_p) || !is.numeric(q_m) || !is.numeric(q_r) || !is.numeric(q_z)){
    stop("All constraints must be integers!\n")
  }
  
  # All column constraints must be numbers.
  if (!is.numeric(n_p) || !is.numeric(n_m) || !is.numeric(n_z)){
    stop("All constraints must be integers!\n")
  }
  
  # All constraints must be non-negative.
  if (q_p < 0 || q_m < 0 || q_r < 0 || q_z < 0 || n_p < 0 || n_m < 0 || n_z < 0){
    stop("All constraints must be non-negative!\n")
  }
  
  # Sum of row constraints must equal sum of column constraints.
  if ((q_p + q_m + q_r + q_z) != (n_p + n_m + n_z)){
    stop("Sum of row constraints must equal sum of column constraints!\n")
  }
  
  # All row constraints must be integers.
  if (floor(q_p) != q_p || floor(q_m) != q_m || floor(q_r) != q_r || floor(q_z) != q_z){
    stop("All constraints must be integers!\n")
  }
  
  # All column constraints must be integers.
  if (floor(n_p) != n_p || floor(n_m) != n_m || floor(n_z) != n_z){
    stop("All constraints must be integers!\n")
  }
  
  domain <- domain_4by3(q_p, q_m, q_r, q_z, n_p, n_m, n_z)

  return(domain)
}



#' Computes the probability mass function of the scores.
#' 
#' @description This function computes the probability mass function for the Quaternary Dot
#' Product Scoring Statistic for signed causal graphs. This includes scores with probabilities strictly 
#' greater than zero. 
#' 
#' @usage QP_Pmf(q_p, q_m, q_z, q_r, n_p, n_m, n_z, epsilon = 1e-16)
#' 
#' @param q_p Expected number of positive predictions.
#' @param q_m Expected number of negative predictions.
#' @param q_z Expected number of nil predictions.
#' @param q_r Expected number of regulated predictions. 
#'
#' @param n_p Number of positive predictions from experiments.
#' @param n_m Number of negative predictions from experiments.
#' @param n_z Number of nil predictions from experiments. 
#' @param epsilon parameter for thresholding probabilities of matrices. Default value is 1e-16.
#' 
#' @return Vector of probabilities for scores in the support.
#' 
#' @author Carl Tony Fakhry, Ping Chen and Kourosh Zarringhalam
#' 
#' @references Carl Tony Fakhry, Parul Choudhary, Alex Gutteridge, Ben Sidders, Ping Chen, Daniel Ziemek, and
#'             Kourosh Zarringhalam. Interpreting transcriptional changes using causal graphs: new methods and
#'             their practical utility on public networks. BMC Bioinformatics, 17:318, 2016. ISSN 1471-2105.
#'             doi: 10.1186/s12859-016-1181-8.
#'            
#'             Franceschini, A (2013). STRING v9.1: protein-protein interaction networks, with increased coverage 
#'             and integration. In:'Nucleic Acids Res. 2013 Jan;41(Database issue):D808-15. doi: 10.1093/nar/gks1094. 
#'             Epub 2012 Nov 29'.
#'             
#' @details This function computes the probability for each score in the support of the distribution.
#'          The returned value is a vector of probabilities where the 
#'          returned vector has names set equal to the corresponding scores.
#' 
#'          Setting epsilon to zero will compute the probability mass function without ignoring any matrices 
#'          with probabilities smaller than epsilon*D_max (D_max is the numerator associated with the matrix of highest
#'          probability for the given constraints). The default value of 1e-16 is experimentally validated to be 
#'          a very reasonable threshold. Setting the threshold to higher values which are smaller than 1 will lead to understimating
#'          the probabilities of each score since more tables will be ignored. 
#' 
#' @examples # Compute the probability mass function of the Quaternary Dot
#' # Product Scoring Statistic for the given table margins.
#' pmf <- QP_Pmf(50,50,50,0,50,50,50)
#' 
#' @seealso \code{\link{QP_Pvalue}}, \code{\link{QP_Support}}
#' 
#' @export

QP_Pmf <- function(q_p, q_m, q_z, q_r, n_p, n_m, n_z, epsilon = 1e-16){
  
  # All row constraints must be numbers.
  if (!is.numeric(q_p) || !is.numeric(q_m) || !is.numeric(q_r) || !is.numeric(q_z)){
    stop("All constraints must be integers!\n")
  }
  
  # All column constraints must be numbers.
  if (!is.numeric(n_p) || !is.numeric(n_m) || !is.numeric(n_z)){
    stop("All constraints must be integers!\n")
  }
  
  # All constraints must be non-negative.
  if (q_p < 0 || q_m < 0 || q_r < 0 || q_z < 0 || n_p < 0 || n_m < 0 || n_z < 0){
    stop("All constraints must be non-negative!\n")
  }
  
  # Epsilon must be positive between 0 and 1.
  if (length(epsilon) != 1 || !is.numeric(epsilon) || epsilon < 0 || epsilon > 1){
    stop("Epsilon must be positive numeric number >= 0 and <= 1!\n")
  }
  
  # Sum of row constraints must equal sum of column constraints.
  if ((q_p + q_m + q_r + q_z) != (n_p + n_m + n_z)){
    stop("Sum of row constraints must equal sum of column constraints!\n")
  }
  
  # All row constraints must be integers.
  if (floor(q_p) != q_p || floor(q_m) != q_m || floor(q_r) != q_r || floor(q_z) != q_z){
    stop("All constraints must be integers!\n")
  }
  
  # All column constraints must be integers.
  if (floor(n_p) != n_p || floor(n_m) != n_m || floor(n_z) != n_z){
    stop("All constraints must be integers!\n")
  }
  
  distribution <- pmf_4by3(q_p, q_m, q_r, q_z, n_p, n_m, n_z, epsilon)

  return(distribution)
}



#' Computes the p-value of a score.
#' 
#' @description This function computes the right sided p-value for the Quaternary 
#'              Dot Product Scoring Statistic.
#' @usage QP_Pvalue(score, q_p, q_m, q_z, q_r, n_p, n_m, n_z, epsilon = 1e-16)  
#' 
#' @param score The score for which the p-value will be computed.
#' @param q_p Expected number of positive predictions.
#' @param q_m Expected number of negative predictions.
#' @param q_z Expected number of nil predictions.
#' @param q_r Expected number of regulated predictions. 
#' @param n_p Number of positive predictions from experiments.
#' @param n_m Number of negative predictions from experiments.
#' @param n_z Number of nil predictions from experiments. 
#' @param epsilon Threshold for probabilities of matrices. Default value is 1e-16.
#' 
#' @return This function returns a numerical value, where the numerical value is the p-value of the score.
#' 
#' @author Carl Tony Fakhry, Ping Chen and Kourosh Zarringhalam
#' 
#' @references Carl Tony Fakhry, Parul Choudhary, Alex Gutteridge, Ben Sidders, Ping Chen, Daniel Ziemek, and
#'             Kourosh Zarringhalam. Interpreting transcriptional changes using causal graphs: new methods and
#'             their practical utility on public networks. BMC Bioinformatics, 17:318, 2016. ISSN 1471-2105.
#'             doi: 10.1186/s12859-016-1181-8.
#'            
#'             Franceschini, A (2013). STRING v9.1: protein-protein interaction networks, with increased coverage 
#'             and integration. In:'Nucleic Acids Res. 2013 Jan;41(Database issue):D808-15. doi: 10.1093/nar/gks1094. 
#'             Epub 2012 Nov 29'.
#'             
#' @details Setting epsilon to zero will compute the probability mass function without ignoring any matrices 
#'          with probabilities smaller than epsilon*D_max (D_max is the numerator associated with the matrix of highest
#'          probability for the given constraints). The default value of 1e-16 is experimentally validated to be 
#'          a very reasonable threshold. Setting the threshold to higher values which are smaller than 1 will lead to understimating
#'          the probabilities of each score since more tables will be ignored. 
#' 
#' @examples # Computing The p-value of score 50 
#' # for the given table margins. 
#' pval <- QP_Pvalue(50,50,50,50,0,50,50,50)
#' 
#' @seealso \code{\link{QP_SigPvalue}}
#' 
#' @export

QP_Pvalue <- function(score, q_p, q_m, q_z, q_r, n_p, n_m, n_z, epsilon = 1e-16){
  
  # All row constraints must be numbers.
  if (!is.numeric(q_p) || !is.numeric(q_m) || !is.numeric(q_r) || !is.numeric(q_z)){
    stop("All constraints must be integers!\n")
  }
  
  # All column constraints must be numbers.
  if (!is.numeric(n_p) || !is.numeric(n_m) || !is.numeric(n_z)){
    stop("All constraints must be integers!\n")
  }
  
  # All constraints must be non-negative.
  if (q_p < 0 || q_m < 0 || q_r < 0 || q_z < 0 || n_p < 0 || n_m < 0 || n_z < 0){
    stop("All constraints must be non-negative!\n")
  }
  
  # Epsilon must be positive between 0 and 1.
  if (length(epsilon) != 1 || !is.numeric(epsilon) || epsilon < 0 || epsilon > 1){
    stop("Epsilon must be positive numeric number >= 0 and <= 1!\n")
  }
  
  # Sum of row constraints must equal sum of column constraints.
  if ((q_p + q_m + q_r + q_z) != (n_p + n_m + n_z)){
    stop("Sum of row constraints must equal sum of column constraints!\n")
  }
  
  # All row constraints must be integers.
  if (floor(q_p) != q_p || floor(q_m) != q_m || floor(q_r) != q_r || floor(q_z) != q_z){
    stop("All constraints must be integers!\n")
  }
  
  # All column constraints must be integers.
  if (floor(n_p) != n_p || floor(n_m) != n_m || floor(n_z) != n_z){
    stop("All constraints must be integers!\n")
  }
  
  pval <- Pvalue_4by3(q_p, q_m, q_r, q_z, n_p, n_m, n_z, score, epsilon)

  return(pval)
}



#' Computes the p-value for a statistically significant score.
#' 
#' @description This function computes the right sided p-value for the Quaternary Dot
#'              Product Scoring Statistic for statistically significant scores.
#' @usage QP_SigPvalue(score, q_p, q_m, q_z, q_r, n_p, n_m, n_z, epsilon = 1e-16, sig_level = 0.05)  
#' 
#' @param score The score for which the p-value will be computed.
#' @param q_p Expected number of positive predictions.
#' @param q_m Expected number of negative predictions.
#' @param q_z Expected number of nil predictions.
#' @param q_r Expected number of regulated predictions. 
#' @param n_p Number of positive predictions from experiments.
#' @param n_m Number of negative predictions from experiments.
#' @param n_z Number of nil predictions from experiments. 
#' @param epsilon Threshold for probabilities of matrices. Default value is 1e-16.
#' @param sig_level Significance level of test hypothesis. Default value is 0.05.
#' 
#' @return This function returns a numerical value, where the numerical value is the p-value of a score if the score
#'         is statistically significant otherwise it returns -1.
#'         
#' 
#' @author Carl Tony Fakhry, Ping Chen and Kourosh Zarringhalam
#' 
#' @references Carl Tony Fakhry, Parul Choudhary, Alex Gutteridge, Ben Sidders, Ping Chen, Daniel Ziemek, and
#'             Kourosh Zarringhalam. Interpreting transcriptional changes using causal graphs: new methods and
#'             their practical utility on public networks. BMC Bioinformatics, 17:318, 2016. ISSN 1471-2105.
#'             doi: 10.1186/s12859-016-1181-8.
#'            
#'             Franceschini, A (2013). STRING v9.1: protein-protein interaction networks, with increased coverage 
#'             and integration. In:'Nucleic Acids Res. 2013 Jan;41(Database issue):D808-15. doi: 10.1093/nar/gks1094. 
#'             Epub 2012 Nov 29'.
#'              
#' @details Setting epsilon to zero will compute the probability mass function without ignoring any matrices 
#'          with probabilities smaller than epsilon*D_max (D_max is the numerator associated with the matrix of highest
#'          probability for the given constraints). The default value of 1e-16 is experimentally validated to be 
#'          a very reasonable threshold. Setting the threshold to higher values which are smaller than 1 will lead to understimating
#'          the probabilities of each score since more tables will be ignored. If the score is not statistically significant, 
#'          then a value of -1 will be returned.
#' 
#' @examples # Computing The p-value of score 50 
#' # for the given table margins. 
#' pval <- QP_SigPvalue(50,50,50,50,0,50,50,50)
#' 
#' @seealso \code{\link{QP_Pvalue}}
#' 
#' @export

QP_SigPvalue <- function(score, q_p, q_m, q_z, q_r, n_p, n_m, n_z, epsilon = 1e-16, sig_level = 0.05){
  
  # All row constraints must be numbers.
  if (!is.numeric(q_p) || !is.numeric(q_m) || !is.numeric(q_r) || !is.numeric(q_z)){
    stop("All constraints must be integers!\n")
  }
  
  # All column constraints must be numbers.
  if (!is.numeric(n_p) || !is.numeric(n_m) || !is.numeric(n_z)){
    stop("All constraints must be integers!\n")
  }
  
  # All constraints must be non-negative.
  if (q_p < 0 || q_m < 0 || q_r < 0 || q_z < 0 || n_p < 0 || n_m < 0 || n_z < 0){
    stop("All constraints must be non-negative!\n")
  }
  
  # Epsilon must be positive between 0 and 1.
  if (length(epsilon) != 1 || !is.numeric(epsilon) || epsilon < 0 || epsilon > 1){
    stop("Epsilon must be positive numeric number >= 0 and <= 1!\n")
  }
  
  # Sum of row constraints must equal sum of column constraints.
  if ((q_p + q_m + q_r + q_z) != (n_p + n_m + n_z)){
    stop("Sum of row constraints must equal sum of column constraints!\n")
  }
  
  # All row constraints must be integers.
  if (floor(q_p) != q_p || floor(q_m) != q_m || floor(q_r) != q_r || floor(q_z) != q_z){
    stop("All constraints must be integers!\n")
  }
  
  # All column constraints must be integers.
  if (floor(n_p) != n_p || floor(n_m) != n_m || floor(n_z) != n_z){
    stop("All constraints must be integers!\n")
  }
  
  pval <- SigPvalue_4by3(q_p, q_m, q_r, q_z, n_p, n_m, n_z, score, epsilon, sig_level)
  
  return(pval)
}



#' Computes the probability of a score.
#' 
#' @description This function computes the probability of a score in the Quaternary
#'              Dot Product scoring distribution.
#' @usage QP_Probability(score, q_p, q_m, q_z, q_r, n_p, n_m, n_z, epsilon = 1e-16)  
#' 
#' @param score The score for which the probability will be computed.
#' @param q_p Expected number of positive predictions.
#' @param q_m Expected number of negative predictions.
#' @param q_z Expected number of nil predictions.
#' @param q_r Expected number of regulated predictions. 
#' @param n_p Number of positive predictions from experiments.
#' @param n_m Number of negative predictions from experiments.
#' @param n_z Number of nil predictions from experiments. 
#' @param epsilon Threshold for probabilities of matrices. Default value is 1e-16.
#' 
#' @return This function returns a numerical value, where the numerical value is the probability of the score.
#' 
#' @author Carl Tony Fakhry, Ping Chen and Kourosh Zarringhalam
#' 
#' @references Carl Tony Fakhry, Parul Choudhary, Alex Gutteridge, Ben Sidders, Ping Chen, Daniel Ziemek, and
#'             Kourosh Zarringhalam. Interpreting transcriptional changes using causal graphs: new methods and
#'             their practical utility on public networks. BMC Bioinformatics, 17:318, 2016. ISSN 1471-2105.
#'             doi: 10.1186/s12859-016-1181-8.
#'            
#'             Franceschini, A (2013). STRING v9.1: protein-protein interaction networks, with increased coverage 
#'             and integration. In:'Nucleic Acids Res. 2013 Jan;41(Database issue):D808-15. doi: 10.1093/nar/gks1094. 
#'             Epub 2012 Nov 29'.
#'              
#' @details Setting epsilon to zero will compute the probability mass function without ignoring any matrices 
#'          with probabilities smaller than epsilon*D_max (D_max is the numerator associated with the matrix of highest
#'          probability for the given constraints). The default value of 1e-16 is experimentally validated to be 
#'          a very reasonable threshold. Setting the threshold to higher values which are smaller than 1 will lead to understimating
#'          the probabilities of each score since more tables will be ignored. 
#'          
#'          For computing p-values, the user is advised to use the p-value function which is optimized
#'          for such purposes.
#' 
#' @examples # Computing The probability of score 50 
#' # for the given table margins. 
#' prob <- QP_Probability(0,50,50,50,0,50,50,50)
#' 
#' @seealso \code{\link{QP_Pmf}}, \code{\link{QP_Pvalue}}, \code{\link{QP_SigPvalue}}
#' 
#' @export

QP_Probability <- function(score, q_p, q_m, q_z, q_r, n_p, n_m, n_z, epsilon = 1e-16){
  
  # All row constraints must be numbers.
  if (!is.numeric(q_p) || !is.numeric(q_m) || !is.numeric(q_r) || !is.numeric(q_z)){
    stop("All constraints must be integers!\n")
  }
  
  # All column constraints must be numbers.
  if (!is.numeric(n_p) || !is.numeric(n_m) || !is.numeric(n_z)){
    stop("All constraints must be integers!\n")
  }
  
  # All constraints must be non-negative.
  if (q_p < 0 || q_m < 0 || q_r < 0 || q_z < 0 || n_p < 0 || n_m < 0 || n_z < 0){
    stop("All constraints must be non-negative!\n")
  }
  
  # Epsilon must be positive between 0 and 1.
  if (length(epsilon) != 1 || !is.numeric(epsilon) || epsilon < 0 || epsilon > 1){
    stop("Epsilon must be positive numeric number >= 0 and <= 1!\n")
  }
  
  # Sum of row constraints must equal sum of column constraints.
  if ((q_p + q_m + q_r + q_z) != (n_p + n_m + n_z)){
    stop("Sum of row constraints must equal sum of column constraints!\n")
  }
  
  # All row constraints must be integers.
  if (floor(q_p) != q_p || floor(q_m) != q_m || floor(q_r) != q_r || floor(q_z) != q_z){
    stop("All constraints must be integers!\n")
  }
  
  # All column constraints must be integers.
  if (floor(n_p) != n_p || floor(n_m) != n_m || floor(n_z) != n_z){
    stop("All constraints must be integers!\n")
  }
  
  prob <- user_probability_of_score_4by3(score, q_p, q_m, q_r, q_z, n_p, n_m, n_z, epsilon)

  return(prob)
}

#' @importFrom stats na.omit complete.cases
#' @importFrom utils read.table setTxtProgressBar txtProgressBar
#' @import dplyr
#' @import yaml
#' @importFrom Rcpp evalCpp 
#' @useDynLib QuaternaryProd
NULL

# library(org.Hs.eg.db)
# library(stringr)
# library(readr)
# 
# # Function to parse the StringDB network
# parseStringDB <- function(){
# 
#   # Get the full file name containing the STRINGdb Rels
#   ff <- system.file("extdata", "9606.protein.actions.v10.txt.gz", package="QuaternaryProd")
#   all_rels <- read_tsv(gzfile(ff), col_names = TRUE)
# 
#   # Set new names for columns
#   names(all_rels) <- c("srcuid", "trguid", "mode", "action", "direction","score")
#   Rels <- all_rels[, c("srcuid", "trguid", "mode", "direction")]
# 
#   # Get all rows with causal Rels
#   Rels <- Rels[Rels$mode %in% c("activation", "inhibition","expression"),]
# 
#   # Get causal Rels where direction is not specified, and consider reversed
#   # direction of causality as a valid causal relation
#   Bidirectional <- Rels[Rels$direction == 0 , c("trguid", "srcuid", "mode", "direction")]
#   names(Bidirectional) <- c("srcuid", "trguid", "mode", "direction")
#   Rels <- bind_rows(Rels, Bidirectional)
#   Rels$direction <- NULL
#   Rels <- unique(Rels)
# 
#   # Rename activation as increases, inhibition as decreases, expression
#   # as regulates
#   Rels$mode <- sub("activation", "increases", Rels$mode)
#   Rels$mode <- sub("inhibition", "decreases", Rels$mode)
#   Rels$mode <- sub("expression", "regulates", Rels$mode)
#   Rels <- unique(Rels)
# 
#   # Get all unique protein ensemble ids in the causal network
#   allEns <- unique(c(Rels$srcuid, Rels$trguid))
# 
#   # Get the entities data frame
#   ff2 <- system.file("extdata", "ENSPid_to_Entrezid_v10.csv", package="QuaternaryProd")
#   Ensemble2Entrez = read.table(ff2, sep = ",", header = T, stringsAsFactors = F)
# 
#   Ents <- data.frame(id = Ensemble2Entrez$Entrez_Gene_ID, ensembleid = Ensemble2Entrez$STRING_Locus_ID, stringsAsFactors = F)
#   map = org.Hs.egSYMBOL
#   symbol <- unlist(mget(as.character(Ents$id), map, ifnotfound=NA))
#   symbol[is.na(symbol)] <- "No-Symbol"
#   Ents["symbol"] <- symbol
# 
#   # Keep entities in the network
#   Ents <- Ents[Ents$ensembleid %in% allEns,]
# 
#   # Remove ensemble ids in entities with duplicated entrez id
#   Ents <- Ents[!duplicated(Ents$id),]
# 
#   allEns2 <- allEns[which(!(allEns %in% Ents$ensembleid))]
#   map <- org.Hs.egENSEMBLPROT2EG
#   x <- unlist(mget(str_replace(allEns2, "9606.",""), map, ifnotfound=NA));
#   Ents2 <- data.frame(ensembleid=paste("9606.", names(x), sep=""), id= as.integer(x), stringsAsFactors = F)
#   Ents2$id[is.na(Ents2$id)] <- -1
#   Ents2$symbol <- unlist(mget(as.character(Ents2$id), org.Hs.egSYMBOL, ifnotfound=NA))
#   Ents2 <- Ents2[Ents2$id != -1,]
#   Ents2 <- Ents2[!duplicated(Ents2$id),]
#   Ents2 <- Ents2[,c("id","ensembleid","symbol")]
#   Ents <- rbind(Ents, Ents2)
#   Ents$id[duplicated(Ents$id)] <- "No-EntrezId"
#   Ents$symbol[duplicated(Ents$symbol)] <- "No-Symbol"
# 
#   allEns2 <- allEns[which(!(allEns %in% Ents$ensembleid))]
#   Ents2 <- data.frame(id = "No-EntrezId", ensembleid = allEns2, symbol = "No-Symbol")
#   Ents <- rbind(Ents, Ents2)
#   Ents <- cbind(uid = 1:nrow(Ents), Ents)
#   Ents <- Ents[!duplicated(Ents$ensembleid),]
# 
#   # Get all unique Rels
#   Rels <- Rels[!(Rels$trguid %in% allEns2),]
#   Rels <- Rels[which(Rels$srcuid %in% Ents$ensembleid & Rels$trguid %in% Ents$ensembleid),]
#   Rels2 <- left_join(Rels, Ents, by = c("srcuid" = "ensembleid")) %>% dplyr::select(srcuid = uid, trguid, mode)
#   Rels2 <- left_join(Rels2, Ents, by = c("trguid" = "ensembleid")) %>% dplyr::select(srcuid, trguid = uid, mode)
#   Rels <- data.frame(Rels2)
#   rm(Rels2)
#   Rels$mode[which(Rels$mode == "increases")] <- 1
#   Rels$mode[which(Rels$mode == "decreases")] <- -1
#   Rels$mode[which(Rels$mode == "regulates")] <- 0
#   Rels$mode <- as.integer(Rels$mode)
#   Rels <- Rels[order(Rels$srcuid, Rels$trguid),]
#   Rels <- unique(Rels)
# 
#   # Leave source proteins which contain at least 10 edges
#   sufficientRels <- group_by(Rels, srcuid) %>% summarise(count=n())
#   sufficientRels <- sufficientRels %>% filter(count > 10)
#   Rels <- Rels %>% filter(srcuid %in% sufficientRels$srcuid)
#   Rels <- Rels[order(Rels$srcuid, Rels$trguid),]
# 
#   # Handle ambiguities if source node affects target node in more than one way
#   indices1 <- which(duplicated(Rels[,c("srcuid", "trguid")]))
#   for(i in indices1){
#     indices2 <- which(Rels$srcuid == Rels$srcuid[i] & Rels$trguid == Rels$trguid[i])
#     if(all(c(-1,1,0) %in% unique(Rels$mode[indices2]))){
#       Rels$mode[indices2] = 0
#     }else if(all(c(-1,1) %in% unique(Rels$mode[indices2]))){
#       Rels$mode[indices2] = 0
#     }else if(1 %in% unique(Rels$mode[indices2])){
#       Rels$mode[indices2] = 1
#     }else if(-1 %in% unique(Rels$mode[indices2])){
#       Rels$mode[indices2] = -1
#     }else{
#       Rels$mode[indices2] = 0
#     }
#   }
#   
#   Rels <- Rels[!duplicated(Rels),]
#   Rels <- Rels[which(Rels$srcuid != Rels$trguid),]
# 
#   write.table(Rels, "/home/kolonel/Programs/QuaternaryProd/inst/extdata/StringRels.dat", col.names = T, row.names = F)
#   write.table(Ents, "/home/kolonel/Programs/QuaternaryProd/inst/extdata/StringEnts.dat", col.names = T, row.names = F)
# 
#   Rels = read.table("~/Programs/QuaternaryProd/inst/extdata/StringRels.dat", header = T)
#   Ents = read.table("~/Programs/QuaternaryProd/inst/extdata/StringEnts.dat", header = T)
#   
#   # Proteins to be tested
#   u.hyps <- unique(Rels$srcuid)
#   
#   # For each protein, get its children
#   child.uid <- lapply(u.hyps, function(x, Rels) Rels$trguid[which(Rels$srcuid == x)], Rels = Rels)
#   
#   # For each protein, Find if its children are upregulated or downregulated
#   child.sgn <- lapply(u.hyps, function(x, Rels) ifelse(Rels$mode[which(Rels$srcuid == x)] == 1,
#                                                             1, ifelse(Rels$mode[which(Rels$srcuid == x)] == -1, -1, 0)), Rels = Rels)
#   
#   write_yaml(u.hyps, file='/home/kolonel/Programs/QuaternaryProd/inst/extdata/u.hyps.yaml')
#   write_yaml(child.uid, file='/home/kolonel/Programs/QuaternaryProd/inst/extdata/child.uid.yaml')
#   write_yaml(child.sgn, file='/home/kolonel/Programs/QuaternaryProd/inst/extdata/child.sgn.yaml')
# 
# }

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

Pvalue_4by3 <- function(q_p, q_m, q_r, q_z, n_p, n_m, n_z, target_score, epsilon) {
    .Call('_QuaternaryProd_Pvalue_4by3', PACKAGE = 'QuaternaryProd', q_p, q_m, q_r, q_z, n_p, n_m, n_z, target_score, epsilon)
}

SigPvalue_4by3 <- function(q_p, q_m, q_r, q_z, n_p, n_m, n_z, target_score, epsilon, significance_level) {
    .Call('_QuaternaryProd_SigPvalue_4by3', PACKAGE = 'QuaternaryProd', q_p, q_m, q_r, q_z, n_p, n_m, n_z, target_score, epsilon, significance_level)
}

domain_4by3 <- function(q_p, q_m, q_r, q_z, n_p, n_m, n_z) {
    .Call('_QuaternaryProd_domain_4by3', PACKAGE = 'QuaternaryProd', q_p, q_m, q_r, q_z, n_p, n_m, n_z)
}

pmf_4by3 <- function(q_p, q_m, q_r, q_z, n_p, n_m, n_z, epsilon) {
    .Call('_QuaternaryProd_pmf_4by3', PACKAGE = 'QuaternaryProd', q_p, q_m, q_r, q_z, n_p, n_m, n_z, epsilon)
}

user_probability_of_score_4by3 <- function(target_score, q_p, q_m, q_r, q_z, n_p, n_m, n_z, epsilon) {
    .Call('_QuaternaryProd_user_probability_of_score_4by3', PACKAGE = 'QuaternaryProd', target_score, q_p, q_m, q_r, q_z, n_p, n_m, n_z, epsilon)
}

---
title: "QuaternaryProd"
author: "Carl Tony Fakhry, Ping Chen and Kourosh Zarringhalam"
date: "`r Sys.Date()`"
output: rmarkdown::pdf_document
urlcolor: blue

vignette: >
  %\VignetteIndexEntry{<span style="color:red">QuaternaryProdVignette</span>}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

\(\indent\) \(\indent\) \(\indent\) \(\indent\) A signed causal graph for gene regulation is a directed graph where the edges are signed and the signs indicate the direction of regulation of the target genes (the signs are either (+) or (-)). **QuaternaryProd** is a package for computing the Quaternary Dot Product Scoring Statistic [2] (or simply the Quaternary Statistic) of signed causal graphs for gene regulation. The Quaternary Dot Product Scoring Statistic is a generalization of the Ternary Dot Product Scoring Statistic [1] which allows for ambiguities to arise in the causal graph. Ambiguities arise when a regulator can affect a target gene in two different ways or if the direction of regulation is unknown. We will first provide some background, and then we will apply the statistic to STRINGdb [3] which is a publicly available biological network.


## Introduction
\(\indent\) \(\indent\) \(\indent\) \(\indent\)  The Quaternary Dot Product Scoring Statistic [2] is a goodness of fit test for evaluating the performance of regulation predictions made by a signed and directed causal network on a given gene expression data set. Given a regulator \(s\) in a causal graph, let \(q_p\), \(q_m\) and \(q_r\) denote the number of target genes which are upregulated (+), downregulated (-) and regulated (r) by the regulator \(s\) respectively. Regulated relations occur when a regulator regulates a target gene without knowing the direction of regulation or if an ambiguity in direction of regulation occurs. An ambiguity can occur if a regulator, according to a given network, shares both (+) and (-) relations with the same target gene. Moreover, let \(q_z\) denote the set of target genes in the causal network which do not share a relation with \(s\) i.e which are not affected by \(s\).
Next, suppose we are presented with new gene expression data. Let \(n_p\), \(n_m\) and \(n_z\) denote the number of genes which are upregulated, downregulated and are unregulated in the gene expression data respectively. For the regulator \(s\), we can tabulate the predictions of the network vs. the gene expression data:

\begin{table}[!htbp]
\centering
\begin{tabular}{l l l l l}
\hline
 & Observed \(+\)  & Observed \(-\) & Observed \(0\) &  Total \\ \hline
Predicted \(+\) & \(n_{pp}\) & \(n_{pm}\)  & \(n_{pz}\)  & \(q_p\) \\
Predicted \(-\) & \(n_{mp}\)  & \(n_{mm}\)  & \(n_{mz}\)  & \(q_m\) \\
Predicted \(r\) & \(n_{rp}\)  & \(n_{rm}\)  & \(n_{rz}\)  & \(q_r\) \\
Predicted \(0\) & \(n_{zp}\) & \(n_{zm}\) & \(n_{zz}\) & \(q_z\) \\
Total & \(n_p\)  & \(n_{m}\)   & \(n_z\) & \(T\) \\
\hline
\end{tabular}
\caption{Tabulation of predictions from network edges vs. observations from experimental results.}
\end{table}

\(n_{pp}\) denotes the number of target genes which \(s\) is predicted to upregulate by the network and were indeed upregulated in the gene expression data. \(n_{pm}\) denotes the number of target genes which \(s\) is predicted to upregulate and were downregulated in the gene expression data. \(n_{pz}\) denotes the number of target genes which \(s\) is predicted to upregulate and were not expressed in the gene expression data. Similar interpretation follows for all other entries of the table. 
The probability of a tabulation table follows the Quaternary Dot Product distribution which is given by:
\begin{align}
P(\text{Table}) &= \frac{ \binom{q_p}{n_{pp},n_{pm},n_{pz}} \binom{q_m}{n_{mp},n_{mm},n_{mz}}\binom{q_z}{n_{zp},n_{zm},n_{zz}}\binom{q_r}{n_{rp},n_{rm},n_{rz}}}{\binom{T}{n_p,n_m,n_z}}.
\end{align}
Note, since the predictions by the network and the experimental values are fixed, then the table has 6 degrees of freedom \(n_{pp}\), \(n_{mm}\), \(n_{rp}\), \(n_{rm}\), \(n_{mp}\) and \(n_{pm}\). The score \(S\) to measure the goodness of fit is given by:
\begin{align}
S(\text{Table}) &= n_{pp} + n_{mm} + n_{rp} + n_{rm} - (n_{mp} + n_{pm}) 
\end{align}
which is the sum of the good predictions (i.e \(n_{pp}\), \(n_{mm}\), \(n_{rp}\) and \(n_{rm}\)) minus the bad predictions (i.e \(n_{mp}\) and \(n_{pm}\)). To compute the probability of a score, we sum the probabilites of all tables with score \(S\) as follows:
\begin{align}
P(S) &= \sum_{P(\text{Table}) = S } P(\text{Table}).
\end{align}

## Functionality
\(\indent\) \(\indent\) \(\indent\) \(\indent\)   **QuaternaryProd** provides different functions for computing the probability of a score, probability mass function, p-value of a score and the domain of the Quaternary Dot Product Scoring Statistic. The probability mass function can be computed if given the margins of the table. 
  
```{R, message=FALSE }
library(QuaternaryProd)

# Compute the probability mass function
pmf <- QP_Pmf(q_p = 20, q_m = 20, q_z = 20, q_r = 0, n_p = 20, n_m = 20, n_z = 20)

# Plot the mass function
plot(names(pmf), pmf, col="blue", xlab = "scores", ylab = "probabilities")
lines(names(pmf), pmf, col = "blue")
```  
  
The package contains optimized functions for computing the p-value of a score. To compute the p-value of score we can use the following:

```{R, message=FALSE }
# Get the p-value of score 5
pval <- QP_Pvalue(score = 5, q_p = 20, q_m = 20, q_z = 20, q_r = 0, 
                                                     n_p = 20, n_m = 20, n_z = 20)
pval

# Compue the p-value only if it is statistically significant otherwise
# return -1
pval <- QP_SigPvalue(score = 5, q_p = 20, q_m = 20, q_z = 20, q_r = 0, 
                                                     n_p = 20, n_m = 20, n_z = 20)
pval
```  

If the user is only interested in obtaining statistically significant p-values, then `QP_SigPvalue` is optimized for this purpose. In either case, the user is advised to compute the p-value of a score using the previous two functions which will be faster than computing the entire probability mass function and then computing the p-value.
Finally, it is possible to also compute the probabilities of scores individually using `QP_Probability` and the support of the distribution using `QP_Support`. 
Since this package is written for the benefit of bioinformaticians, we will provide an example on how to apply this statistic to a publicly available network.
One bioinformatic application is to test how well protein-protein causal networks can predict the regulators in gene expression data. In the last section of this Vignette, we present an example for computing this statistic over the STRINGdb network.   
  
## Functionality for working with the Homo sapien causal network from STRINGdb

\(\indent\) \(\indent\) \(\indent\) \(\indent\) Here we provide functionality for using **QuaternaryProd** with the STRINGdb Homo Sapien causal network (version 10) provided under the [creative commons license](https://creativecommons.org/licenses/by/3.0/).


### Compute Pvalues Over the Network
\indent  Given new gene expression data, we can compute the scores and p-values for all regulators in the STRINGdb Homo Sapien causal network using the specialized `RunCRE_HSAStringDB` function. Note that the user can specify and use other causal networks with `RunCRE_HSAStringDB`. We use the gene expression data sets that were used in [1]. The data sets contain the c-Myc and E2F3 expression signatures. Note that the results may differ from those reported in [2] since the network was parsed differently.

#### Load Gene Expression Data
First, we load all the data sets. The gene expression data sets must have the following columns: 1- `entrez` column corresponding to the entrez id of the gene, 2- `pvalue` column corresponding to the pvalue of the gene, 3- `fc` column corresponding to the fold change of the gene. After we load the data sets, we make sure that there are no duplicated entrez ids in the data sets.


```{r, message=FALSE}
library(QuaternaryProd)

# Get gene expression data
e2f3 <- system.file("extdata", "e2f3_sig.txt",
                             package = "QuaternaryProd")
e2f3 <- read.table(e2f3, sep = "\t",
                             header = TRUE, stringsAsFactors = FALSE)
myc <- system.file("extdata", "myc_sig.txt",
                             package = "QuaternaryProd")
myc <- read.table(myc, sep = "\t",
                             header = TRUE, stringsAsFactors = FALSE)

# Rename column names appropriately
# and remove duplicated entrez ids in the gene expression data
names(e2f3) <- c("entrez", "pvalue", "fc")
e2f3 <- e2f3[!duplicated(e2f3$entrez),]

names(myc) <- c("entrez", "pvalue", "fc")
myc <- myc[!duplicated(myc$entrez),]
```

#### Compute the Quaternary Dot Product Scoring Statistic over STRINGdb

We can now compute the Quaternary Dot Product Scoring Statistic over STRINGdb using the following:

```{r, message=FALSE, progress=FALSE}
# Compute the Quaternary Dot Product Scoring Statistic for only statistically
# significant regulators
quaternary_results <- RunCRE_HSAStringDB(e2f3, method = "Quaternary",
                                   fc.thresh = log2(1.3), pval.thresh = 0.05,
                                   only.significant.pvalues = TRUE,
                                   significance.level = 0.05,
                                   epsilon = 1e-16, progressBar = FALSE,
                                   relations = NULL, entities = NULL)
quaternary_results[1:4, c("uid","symbol","regulation","pvalue")]
```
<br/>
<br/>
`RunCRE_HSAStringDB` returns a data frame containing all the regulators of the String causal network. An efficient algorithm for computing the Quaternary Dot Product Scoring Statistic is presented in [2] and is implemented in **QuaternaryProd**. To further improve performance, `RunCRE_HSAStringDB` has an optional argument `only.significant.pvalues` which can be set to `TRUE` so that only the p-values of the statistically significant regulators are computed. P-values which are not statistically significant (i.e p-values greater than `signficance.level`) are not reported, in which case the p-value is set to a value of -1. If the user wishes to compute p-values for regulators which are not statistically significant then the user should set the parameter `only.significant.pvalues = FALSE`. The regulators are ordered in increasing order of the p-values (Note: details on the columns of the data frame returned can be found in the help page for `RunCRE_HSAStringDB`). Finally, we see that this approach retrieves the signal regulator E2F1. 

#### Compute the Ternary Dot Product Scoring Statistic and the Enrichment test over STRINGdb

To compute the Ternary Dot Product Statistic over STRINGdb we can use the following:

```{r, message=FALSE, progress=FALSE}
ternary_results <- RunCRE_HSAStringDB(myc, method = "Ternary",
                                      fc.thresh = log2(1.3), pval.thresh = 0.05,
                                      only.significant.pvalues = TRUE,
                                      significance.level = 0.05,
                                      epsilon = 1e-16, progressBar = FALSE,
                                      relations = NULL, entities = NULL)
ternary_results[1:4, c("uid","symbol","regulation","pvalue")]
```
<br/>
<br/>

We see that this method retrieves MYC as a significant regulator. To compute the Enrichment test over STRINGdb we can use the following:

```{r, message=FALSE, progress=TRUE, quiet = TRUE}
enrichment_results <- RunCRE_HSAStringDB(myc, method = "Enrichment",
                                         fc.thresh = log2(1.3), pval.thresh = 0.05,
                                         only.significant.pvalues = TRUE,
                                         significance.level = 0.05,
                                         epsilon = 1e-16, progressBar = FALSE,
                                         relations = NULL, entities = NULL)
enrichment_results[enrichment_results$symbol == "MYC", c("symbol","regulation","pvalue")]
```
<br/>
<br/>

We see that the Enrichment method also retrieves MYC as a significant regulator although not as significant as in the case of the Ternary Dot Product Scoring Statistic.


## References

[1] Chindelevitch et al. (2012). Assessing statistical significance in causal graphs. BMC Bioinformatics, Volume 3, Issue 1, 2012, Page 35.

[2] Carl Tony Fakhry, Parul Choudhary, Alex Gutteridge, Ben Sidders, Ping Chen, Daniel Ziemek, and Kourosh Zarringhalam. Interpreting transcriptional changes using causal graphs: new methods and their practical utility on public networks. BMC Bioinformatics, 17:318, 2016. ISSN 1471-2105. doi: 10.1186/s12859-016-1181-8.

[3] Franceschini, A (2013). STRING v9.1: protein-protein interaction networks, with increased coverage and integration. In:'Nucleic Acids Res. 2013 Jan;41(Database issue):D808-15. doi: 10.1093/nar/gks1094. Epub 2012 Nov 29'.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UserFunctions.R
\name{QP_Pvalue}
\alias{QP_Pvalue}
\title{Computes the p-value of a score.}
\usage{
QP_Pvalue(score, q_p, q_m, q_z, q_r, n_p, n_m, n_z, epsilon = 1e-16)
}
\arguments{
\item{score}{The score for which the p-value will be computed.}

\item{q_p}{Expected number of positive predictions.}

\item{q_m}{Expected number of negative predictions.}

\item{q_z}{Expected number of nil predictions.}

\item{q_r}{Expected number of regulated predictions.}

\item{n_p}{Number of positive predictions from experiments.}

\item{n_m}{Number of negative predictions from experiments.}

\item{n_z}{Number of nil predictions from experiments.}

\item{epsilon}{Threshold for probabilities of matrices. Default value is 1e-16.}
}
\value{
This function returns a numerical value, where the numerical value is the p-value of the score.
}
\description{
This function computes the right sided p-value for the Quaternary 
             Dot Product Scoring Statistic.
}
\details{
Setting epsilon to zero will compute the probability mass function without ignoring any matrices 
         with probabilities smaller than epsilon*D_max (D_max is the numerator associated with the matrix of highest
         probability for the given constraints). The default value of 1e-16 is experimentally validated to be 
         a very reasonable threshold. Setting the threshold to higher values which are smaller than 1 will lead to understimating
         the probabilities of each score since more tables will be ignored.
}
\examples{
# Computing The p-value of score 50 
# for the given table margins. 
pval <- QP_Pvalue(50,50,50,50,0,50,50,50)

}
\references{
Carl Tony Fakhry, Parul Choudhary, Alex Gutteridge, Ben Sidders, Ping Chen, Daniel Ziemek, and
            Kourosh Zarringhalam. Interpreting transcriptional changes using causal graphs: new methods and
            their practical utility on public networks. BMC Bioinformatics, 17:318, 2016. ISSN 1471-2105.
            doi: 10.1186/s12859-016-1181-8.
           
            Franceschini, A (2013). STRING v9.1: protein-protein interaction networks, with increased coverage 
            and integration. In:'Nucleic Acids Res. 2013 Jan;41(Database issue):D808-15. doi: 10.1093/nar/gks1094. 
            Epub 2012 Nov 29'.
}
\seealso{
\code{\link{QP_SigPvalue}}
}
\author{
Carl Tony Fakhry, Ping Chen and Kourosh Zarringhalam
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UserFunctions.R
\name{QP_Pmf}
\alias{QP_Pmf}
\title{Computes the probability mass function of the scores.}
\usage{
QP_Pmf(q_p, q_m, q_z, q_r, n_p, n_m, n_z, epsilon = 1e-16)
}
\arguments{
\item{q_p}{Expected number of positive predictions.}

\item{q_m}{Expected number of negative predictions.}

\item{q_z}{Expected number of nil predictions.}

\item{q_r}{Expected number of regulated predictions.}

\item{n_p}{Number of positive predictions from experiments.}

\item{n_m}{Number of negative predictions from experiments.}

\item{n_z}{Number of nil predictions from experiments.}

\item{epsilon}{parameter for thresholding probabilities of matrices. Default value is 1e-16.}
}
\value{
Vector of probabilities for scores in the support.
}
\description{
This function computes the probability mass function for the Quaternary Dot
Product Scoring Statistic for signed causal graphs. This includes scores with probabilities strictly 
greater than zero.
}
\details{
This function computes the probability for each score in the support of the distribution.
         The returned value is a vector of probabilities where the 
         returned vector has names set equal to the corresponding scores.

         Setting epsilon to zero will compute the probability mass function without ignoring any matrices 
         with probabilities smaller than epsilon*D_max (D_max is the numerator associated with the matrix of highest
         probability for the given constraints). The default value of 1e-16 is experimentally validated to be 
         a very reasonable threshold. Setting the threshold to higher values which are smaller than 1 will lead to understimating
         the probabilities of each score since more tables will be ignored.
}
\examples{
# Compute the probability mass function of the Quaternary Dot
# Product Scoring Statistic for the given table margins.
pmf <- QP_Pmf(50,50,50,0,50,50,50)

}
\references{
Carl Tony Fakhry, Parul Choudhary, Alex Gutteridge, Ben Sidders, Ping Chen, Daniel Ziemek, and
            Kourosh Zarringhalam. Interpreting transcriptional changes using causal graphs: new methods and
            their practical utility on public networks. BMC Bioinformatics, 17:318, 2016. ISSN 1471-2105.
            doi: 10.1186/s12859-016-1181-8.
           
            Franceschini, A (2013). STRING v9.1: protein-protein interaction networks, with increased coverage 
            and integration. In:'Nucleic Acids Res. 2013 Jan;41(Database issue):D808-15. doi: 10.1093/nar/gks1094. 
            Epub 2012 Nov 29'.
}
\seealso{
\code{\link{QP_Pvalue}}, \code{\link{QP_Support}}
}
\author{
Carl Tony Fakhry, Ping Chen and Kourosh Zarringhalam
}

\name{QuaternaryProd-package}
\alias{QuaternaryProd-package}
%\alias{QuaternaryProd}
\docType{package}
\title{
Computes the Quaternary Dot Product Scoring Statistic for Signed and Unsigned Causal Graphs
}
\description{
	     QuaternaryProd is an R package that performs causal reasoning on biological 
             networks, including publicly available networks such as STRINGdb. QuaternaryProd 
             is an open-source alternative to commercial products such as Inginuity Pathway 
             Analysis. For a given a set of differentially expressed genes, QuaternaryProd 
             computes the significance of upstream regulators in the network by performing causal
             reasoning using the Quaternary Dot Product Scoring Statistic (Quaternary Statistic),
             Ternary Dot product Scoring Statistic (Ternary Statistic) and Fisher's exact test
	     (Enrichment test). The Quaternary Statistic handles signed, unsigned and ambiguous
             edges in the network. Ambiguity arises when the direction of causality is unknown,
             or when the source node (e.g., a protein) has edges with conflicting signs for the 
	     same target gene. On the other hand, the Ternary Statistic provides causal reasoning
             using the signed and unambiguous edges only. The Vignette provides more details on the
             Quaternary Statistic and illustrates an example of how to perform causal reasoning
             using STRINGdb. 
}
\details{
\tabular{ll}{
Package: \tab QuaternaryProd\cr
Type: \tab Package\cr
Version: \tab 1.15.3\cr
Date: \tab 2015-10-22\cr
License: \tab GPL (>= 2)\cr
}

}
\author{
Carl Tony Fakhry, Ping Chen and Kourosh Zarringhalam 

Maintainer: Carl Tony Fakhry <cfakhry@cs.umb.edu>
}
\references{
Carl Tony Fakhry, Parul Choudhary, Alex Gutteridge, Ben Sidders, Ping Chen, Daniel Ziemek, and Kourosh Zarringhalam. Interpreting transcriptional changes using causal graphs: new methods and their practical utility on public networks. BMC Bioinformatics, 17:318, 2016. ISSN 1471-2105. doi: 10.1186/s12859-016-1181-8.

Franceschini, A (2013). STRING v9.1: protein-protein interaction networks, with increased coverage and integration. In:'Nucleic Acids Res. 2013 Jan;41(Database issue):D808-15. doi: 10.1093/nar/gks1094. Epub 2012 Nov 29'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UserFunctions.R
\name{QP_Support}
\alias{QP_Support}
\title{Computes the support for the scores.}
\usage{
QP_Support(q_p, q_m, q_z, q_r, n_p, n_m, n_z)
}
\arguments{
\item{q_p}{Expected number of positive predictions.}

\item{q_m}{Expected number of negative predictions.}

\item{q_z}{Expected number of nil predictions.}

\item{q_r}{Expected number of regulated predictions.}

\item{n_p}{Number of positive predictions from experiments.}

\item{n_m}{Number of negative predictions from experiments.}

\item{n_z}{Number of nil predictions from experiments.}
}
\value{
Integer vector of support.
}
\description{
This function computes the support of the Quaternary Dot Product Scoring distribution
for signed causal graphs. This includes all scores which have probabilities strictly greater than 0.
}
\examples{
# Compute the support of the Quaternary Dot Product Scoring distribution with the given margins.
QP_Support(50,50,50,0,50,50,50)

}
\references{
Carl Tony Fakhry, Parul Choudhary, Alex Gutteridge, Ben Sidders, Ping Chen, Daniel Ziemek, and
            Kourosh Zarringhalam. Interpreting transcriptional changes using causal graphs: new methods and
            their practical utility on public networks. BMC Bioinformatics, 17:318, 2016. ISSN 1471-2105.
            doi: 10.1186/s12859-016-1181-8.
           
            Franceschini, A (2013). STRING v9.1: protein-protein interaction networks, with increased coverage 
            and integration. In:'Nucleic Acids Res. 2013 Jan;41(Database issue):D808-15. doi: 10.1093/nar/gks1094. 
            Epub 2012 Nov 29'.
}
\author{
Carl Tony Fakhry, Ping Chen and Kourosh Zarringhalam
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ProcessData.R
\name{RunCRE_HSAStringDB}
\alias{RunCRE_HSAStringDB}
\title{This function runs a causal relation engine by computing the Quaternary Dot
Product Scoring Statistic, Ternary Dot Product Scoring Statistic or the Enrichment test over the Homo
Sapien STRINGdb causal network (version 10 provided under the Creative Commons license: 
https://creativecommons.org/licenses/by/3.0/). Note that the user has the option of specifying other causal networks
with this function.}
\usage{
RunCRE_HSAStringDB(gene_expression_data, method = "Quaternary", 
                    fc.thresh = log2(1.3), pval.thresh = 0.05, 
                    only.significant.pvalues = FALSE, 
                    significance.level = 0.05,
                    epsilon = 1e-16, progressBar = TRUE, 
                    relations = NULL, entities = NULL)
}
\arguments{
\item{gene_expression_data}{A data frame for gene expression data. The \code{gene_expression_data} data frame must have three columns \code{entrez}, 
\code{fc} and \code{pvalue}. \code{entrez} denotes the entrez id of a given gene, \code{fc} denotes
the fold change of a gene, and \code{pvalue} denotes the p-value. The \code{entrez} column must be of type
integer or character, and the \code{fc} and \code{pvalue} columns must be numeric values.}

\item{method}{Choose one of \code{Quaternary}, \code{Ternary} or \code{Enrichment}. Default is \code{Quaternary}.}

\item{fc.thresh}{Threshold for fold change in \code{gene_expression_data} data frame. Any row in gene_expression_data with abosolute value of \code{fc}
smaller than \code{fc.thresh} will be ignored. Default value is \code{fc.thresh = log2(1.3)}.}

\item{pval.thresh}{Threshold for p-values in \code{gene_expression_data} data frame. All rows in \code{gene_expression_data} with p-values 
greater than \code{pval.thresh} will be ingnored. Default value is \code{pval.thresh = 0.05}.}

\item{only.significant.pvalues}{If \code{only.significant.pvalues = TRUE} then only p-values for statistically significant regulators
are computed otherwise uncomputed p-values are set to -1. The default value is \code{only.significant.pvalues = FALSE}.}

\item{significance.level}{When \code{only.significant.pvalues = TRUE}, only p-values which are less than or equal to 
\code{significance.level} are computed. The default value is \code{significance.level = 0.05}.}

\item{epsilon}{Threshold for probabilities of matrices. Default value is \code{threshold = 1e-16}.}

\item{progressBar}{Progress bar for the percentage of computed p-values for the regulators in the network. Default
value is \code{progressBar = TRUE}.}

\item{relations}{A data frame containing pairs of connected entities in a causal network,
and the type of causal relation between them. The data frame must have three columns with column names: \emph{srcuid}, 
\emph{trguid} and \emph{mode} respective of order. \emph{srcuid} stands for source entity, \emph{trguid} stands for 
target entity and \emph{mode} stands for the type of relation between \emph{srcuid} and \emph{trguid}. The relation 
has to be one of \emph{+1} for \emph{upregulation}, \emph{-1} for \emph{downregulation} or \emph{0} for regulation without
specified direction of regulation. All three columns must be of type integer. Default value is \code{relations = NULL}.}

\item{entities}{A data frame of mappings for all entities present in data frame \emph{relations}. \emph{entities} must contain
four columns: \emph{uid}, \emph{id}, \emph{symbol} and \emph{type} respective of order. \emph{uid} must be 
of type integer and \emph{id}, \emph{symbol} and \emph{type} must be of type character. \emph{uid} includes every source and target 
node in the network (i.e \emph{relations}),
\emph{id} is the id of \emph{uid} (e.g entrez id of an mRNA), \emph{symbol} is the symbol of \emph{id} and \emph{type} 
is the type of entity of \emph{id} (e.g mRNA, protein, drug or compound). Default value is \code{entities = NULL}.}
}
\value{
This function returns a data frame containing parameters concerning the method used. The p-values of each
        of the regulators is also computed, and the data frame
        is in increasing order of p-values of the goodness of fit score for the given regulators. The column
        names of the data frame are:
        
\itemize{        
\item  \code{uid} The regulator in the causal network.
\item \code{symbol} Symbol of the regulator. 
\item \code{regulation} Direction of regulation of the regulator.
\item \code{correct.pred} Number of correct predictions in \code{gene_expression_data} when compared to predictions made
                    by the network.
\item \code{incorrect.pred} Number of incorrect predictions in \code{gene_expression_data} when compared to predictions made
                    by the network.
\item \code{score} The number of correct predictions minus the number of incorrect predictions. 
\item \code{total.reachable} Total Number of children of the given regulator.
\item \code{significant.reachable} Number of children of the given regulator that are also present 
                                   in \code{gene_expression_data}.
\item \code{total.ambiguous} Total number of children of the given regulator which are regulated by the given regulator without
                             knowing the direction of regulation.
\item \code{significant.ambiguous} Total number of children of the given regulator which are regulated by the given regulator without
                             knowing the direction of regulation and are also present in \code{gene_expression_data}.  
\item \code{unknown} Number of target nodes in the causal network which do not interact with the given regulator.
\item \code{pvalue} P-value of the score computed according to the selected method. If \code{only.significant.pvalues = TRUE}
                    and the \code{pvalue} of the regulator is greater than \code{significance.level}, then
                    the p-value is not computed and is set to a value of -1.
}
}
\description{
This function runs a causal relation engine by computing the Quaternary Dot
             Product Scoring Statistic, Ternary Dot Product Scoring Statistic or the Enrichment test over the Homo
             Sapien STRINGdb causal network (version 10 provided under the Creative Commons license: 
             https://creativecommons.org/licenses/by/3.0/). Note that the user has the option of specifying other causal networks
             with this function.
}
\examples{

# Get gene expression data
e2f3 <- system.file("extdata", "e2f3_sig.txt", package = "QuaternaryProd")
e2f3 <- read.table(e2f3, sep = "\\t", header = TRUE, stringsAsFactors = FALSE)

# Rename column names appropriately and remove duplicated entrez ids
names(e2f3) <- c("entrez", "pvalue", "fc")
e2f3 <- e2f3[!duplicated(e2f3$entrez),]

# Compute the Quaternary Dot Product Scoring statistic for statistically significant
# regulators in the STRINGdb network
enrichment_results <- RunCRE_HSAStringDB(e2f3, method = "Enrichment",
                             fc.thresh = log2(1.3), pval.thresh = 0.05,
                             only.significant.pvalues = TRUE)
enrichment_results[1:4, c("uid","symbol","regulation","pvalue")]

}
\references{
Carl Tony Fakhry, Parul Choudhary, Alex Gutteridge, Ben Sidders, Ping Chen, Daniel Ziemek, and
            Kourosh Zarringhalam. Interpreting transcriptional changes using causal graphs: new methods and
            their practical utility on public networks. BMC Bioinformatics, 17:318, 2016. ISSN 1471-2105.
            doi: 10.1186/s12859-016-1181-8.
           
            Franceschini, A (2013). STRING v9.1: protein-protein interaction networks, with increased coverage 
            and integration. In:'Nucleic Acids Res. 2013 Jan;41(Database issue):D808-15. doi: 10.1093/nar/gks1094. 
            Epub 2012 Nov 29'.
}
\author{
Carl Tony Fakhry, Ping Chen and Kourosh Zarringhalam
}
